'\"
'\" Generated from file '../modules/doctools/doctools.man' by tcllib/doctools with format 'nroff'
'\" Copyright (c) 2003 Andreas Kupries <andreas_kupries@users.sourceforge.net>
'\"
'\" -*- tcl -*- doctools manpage
.TH "doctools" n 1.0 doctools "Documentation tools"
.BS
.SH "NAME"
doctools \- Create and manipulate doctools converter object
.SH "SYNOPSIS"
package require \fBTcl  8.2\fR
.sp
package require \fBdoctools  ?1.0?\fR
.sp
\fB::doctools::new\fR \fIobjectName\fR ?\fIoption value\fR...?\fR
.sp
\fB::doctools::help\fR \fR
.sp
\fB::doctools::search\fR \fIpath\fR\fR
.sp
\fBobjectName\fR \fIoption\fR ?\fIarg arg ...\fR?\fR
.sp
\fIobjectName\fR \fBconfigure\fR\fR
.sp
\fIobjectName\fR \fBconfigure\fR \fIoption\fR\fR
.sp
\fIobjectName\fR \fBconfigure\fR \fIoption value\fR...\fR
.sp
\fIobjectName\fR \fBcget\fR \fIoption\fR\fR
.sp
\fIobjectName\fR \fBdestroy\fR\fR
.sp
\fIobjectName\fR \fBformat\fR \fItext\fR\fR
.sp
\fIobjectName\fR \fBsearch\fR \fIpath\fR\fR
.sp
\fIobjectName\fR \fBwarnings\fR\fR
.sp
.BE
.SH "DESCRIPTION"
This package provides objects which can be used to convert text
written in the doctools format as specified in \fBdtformat(n)\fR
into any output format X, assuming that a formatting engine for X is
available and provides the interface specified in
\fBdtformatter(n)\fR.
.SH "API"
.TP
\fB::doctools::new\fR \fIobjectName\fR ?\fIoption value\fR...?\fR
Creates a new doctools object with an associated global Tcl command
whose name is \fIobjectName\fR. This command is explained in full
detail in the sections \fBOBJECT COMMAND\fR and
\fBOBJECT METHODS\fR.
.sp
The list of options and values coming after the name of the object is
used to set the initial configuration of the object.
.TP
\fB::doctools::help\fR \fR
This is a pure convenience command for applications which want to
provide their user with a reminder of the available formatting
commands and their meanings. It returns a string containing a standard
help for this purpose.
.TP
\fB::doctools::search\fR \fIpath\fR\fR
Whenever the package has to map the name of a format to the file
containing the code for its formatting engine it will search the file
in a number of directories. Three such directories are declared by the
package itself.
.sp
However the list is extensible by the user of the package and the
command above is the means to do so. When given a \fIpath\fR to an
existing and readable directory it will prepend that directory to the
existing list. This means that the path added last is searched through
first.
.sp
An error will be thrown if the \fIpath\fR either does not excist, is
not a directory, or is not readable.
.SH "OBJECT COMMAND"
All commands created by \fB::doctools::new\fR have the following
general form and may be used to invoke various operations on the
object they are associated with.
.TP
\fBobjectName\fR \fIoption\fR ?\fIarg arg ...\fR?\fR
The \fIoption\fR and its \fIarg\fRs determine the exact behavior of
the command. See section \fBOBJECT METHODS\fR for more
explanations.
.SH "OBJECT METHODS"
.TP
\fIobjectName\fR \fBconfigure\fR\fR
When called without argument this method returns a list of all known
options and their current values.
.TP
\fIobjectName\fR \fBconfigure\fR \fIoption\fR\fR
When called with a single argument this method behaves like
\fBcget\fR.
.TP
\fIobjectName\fR \fBconfigure\fR \fIoption value\fR...\fR
When called with more than one argument the method reconfigures the
object using the \fIoption\fRs and \fIvalue\fRs given to it.
.sp
The legal configuration options are described in section
\fBOBJECT CONFIGURATION\fR.
.TP
\fIobjectName\fR \fBcget\fR \fIoption\fR\fR
This method expects a legal configuration option as argument and
returns the current value of that option for the object the method was
invoked for.
.sp
The legal configuration options are described in section
\fBOBJECT CONFIGURATION\fR.
.TP
\fIobjectName\fR \fBdestroy\fR\fR
Destroys the object it is invoked for.
.TP
\fIobjectName\fR \fBformat\fR \fItext\fR\fR
Takes the \fItext\fR and runs it through the configured formatting
engine. The resulting string is returned as the result of this
method. An error will be thrown if no \fB-format\fR was configured
for the object.
.sp
The method assumes that the \fItext\fR is in doctools format as
specified in \fBdtformat(n)\fR. Errors will be thrown otherwise.
.TP
\fIobjectName\fR \fBsearch\fR \fIpath\fR\fR
This method extends the per-object list of paths searched for
formatting engines. See also \fB::doctools::search\fR on how to extend
the global (per-package) list of paths.
.sp
The path entered last is searched through first.
.TP
\fIobjectName\fR \fBwarnings\fR\fR
Returns a list containing all the warnings generated by the engine
during the last invocation of method \fBformat\fR.
.SH "OBJECT CONFIGURATION"
All doctools objects understand the following configuration options:
.TP
\fB-file\fR \fIfile\fR
The argument of this option is stored in the object and can be
retrieved by the formatting engine via the command \fBdt_file\fR (see
\fBdtformatter(n)\fR). Its default value is the empty string.
.sp
It will be interpreted as the name of the file containing the text
currently processed by the engine.
.TP
\fB-module\fR \fItext\fR
The argument of this option is stored in the object and can be
retrieved by the formatting engine via the command \fBdt_module\fR
(see \fBdtformatter(n)\fR). Its default value is the empty string.
.sp
It will be interpreted as the name of the module the file containing
the text currently processed by the engine belongs to.
.TP
\fB-format\fR \fItext\fR
The argument of this option specifies the format and thus the engine
to use when converting text via \fBformat\fR. Its default value is
the empty string. No formatting is possible if this
option is not set at least once.
.sp
The package will immediately try to map the name of the format to a
file containing the implementation of the engine for that format. An
error will be thrown if this mapping fails and a previously configured
format is left untouched.
.sp
Section \fBFORMAT MAPPING\fR explains how
the package looks for engine implementations.
.TP
\fB-deprecated\fR \fIboolean\fR
This option is a flag. If set the object will generate warnings when
formatting a text containing the deprecated markup command \fBstrong\fR
Its default value is \fBFALSE\fR. In other words, no warnings will
be generated.
.SH "FORMAT MAPPING"
When trying to map a format name \fIfoo\fR to the file containing
the implementation of formatting engine for \fIfoo\fR the package
will perform the following algorithm:
.IP [1]
If \fIfoo\fR is the name of an existing file this file is directly
taken as the implementation.
.IP [2]
If not, the list of per-object search paths is searched. For each
directory in the list the package checks if that directory contains a
file "\fIfmt.\fIfoo\fR\fR". If yes, that file is taken as the
implementation.
.sp
This list of paths is initially empty and can be extended through the
object method \fBsearch\fR.
.IP [3]
If not, the list of global (package) paths is searched. For each
directory in the list the package checks if that directory contains a
file "\fIfmt.\fIfoo\fR\fR". If yes, that file is taken as the
implementation.
.sp
This list of paths contains initially one path and can be extended
through the command \fB::doctools::search\fR.
.sp
The initial (standard) path is the sub directory "\fImpformats\fR" of
the directory the package itself is located in. In other words, if the
package implementation "\fIdoctools.tcl\fR" is installed in the
directory "\fI/usr/local/lib/tcllib/doctools\fR" then it will by
default search the directory
"\fI/usr/local/lib/tcllib/doctools/mpformats\fR" for format
implementations.
.IP [4]
The mapping fails.
.SH "ENGINES"
The package comes with the following predefined formatting engines
.TP
html
This engine generates HTML markup, for processing by web browsers and
the like.
.TP
latex
This engine generates output suitable for the \fBlatex\fR text
processor coming out of the TeX world.
.TP
list
This engine retrieves version, section and title of the manpage from
the document. As such it can be used to generate a directory listing
for a set of manpages.
.TP
nroff
This engine generates nroff output, for processing by \fBnroff\fR,
or \fBgroff\fR. The result will be standard man pages as they are
known in the unix world.
.TP
null
This engine generates no outout at all. This can be used if one just
wants to validate some input.
.TP
tmml
This engine generates TMML markup as specified by Joe English. The Tcl
Manpage Markup Language is a derivate of XML.
.TP
wiki
This engine generates Wiki markup as understood by Jean Claude
Wippler's \fBwikit\fR application.
.SH "SEE ALSO"
dtformat(n), dtformatter(n), mpexpand(n)
.SH "KEYWORDS"
manpage, TMML, HTML, nroff, conversion, markup
.SH "COPYRIGHT"
.nf
Copyright (c) 2003 Andreas Kupries <andreas_kupries@users.sourceforge.net>
