'\"
'\" Generated from file '../modules/doctools/dtformatter.man' by tcllib/doctools with format 'nroff'
'\" Copyright (c) 2002 Andreas Kupries <andreas_kupries@users.sourceforge.net>
'\" Copyright (c) 2003 Andreas Kupries <andreas_kupries@users.sourceforge.net>
'\"
'\" -*- tcl -*- doctools manpage
.TH "dtformatter" n 1.0 doctools "Documentation tools"
.BS
.SH "NAME"
dtformatter \- Interface specification for formatter code
.SH "DESCRIPTION"
.PP
This manpage specifies the interface between the manpage processor
\fBdoctools::format\fR and the tcl code implementing the conversion
into a particular output format, i.e. the formatting engine.
.PP
The formatting engine has to provide
.IP [1]
Implementations of all the commands specified in \fBdtformat(n)\fR,
using the defined names, but prefixed with the string \fBfmt_\fR,
except for \fBvset\fR and \fBinclude\fR. These two commands are
processed by the generic layer and never seen by the formatting
engine.
.IP [2]
and additionally implementations for
.RS
.TP
\fBfmt_numpasses\fR
This command is called immediately after the formatter is loaded and
has to return the number of passes required by this formatter to
process a manpage. This information has to be an integer number
greater or equal to one.
.TP
\fBfmt_initialize\fR
This command is called at the beginning of every conversion run and is
responsible for initializing the general state of the formatting
engine.
.TP
\fBfmt_setup\fR \fIn\fR
This command is called at the beginning of each pass over the input
and is given the id of the current pass as its first argument. It is
responsible for setting up the internal state of the formatting for
this particular pass.
.TP
\fBfmt_postprocess\fR \fItext\fR
This command is called immediately after the last pass, with the
expansion result of that pass as argument, and can do any last-ditch
modifications of the generated result.  Its result will be the final
result of the conversion.
.sp
Most formats will use \fIidentity\fR here.
.TP
\fBfmt_shutdown\fR
This command is called at the end of every conversion run and is
responsible for cleaning up of all the state in the formatting engine.
.TP
\fBfmt_plain_text\fR \fItext\fR
This command is called for any plain text encountered by the processor
in the input and can do any special processing required for plain
text. Its result is the string written into the expansion.
.sp
Most formats will use \fIidentity\fR here.
.RE
.PP
The tcl code of a formatting engine implementing all of the above can
make the following assumptions about its environment
.IP [1]
It has full access to its own safe interpreter.  In other words, the
engine cannot damage the other parts of the processor, nor can it
damage the filesystem.
.IP [2]
The surrounding system provides the engine with the following
commands:
.RS
.TP
Doctools commands
.RS
.TP
\fBdt_file\fR
Returns the full name of the file currently processed by the engine.
.TP
\fBdt_fileid\fR
Returns the name of the file currently processed by the engine,
without path, nor extension
.TP
\fBdt_format\fR
Returns the name of format loaded into the engine
.TP
\fBdt_lnesting\fR
Returns the number lists currently open
.TP
\fBdt_module\fR
Returns the name of the module the file currently processed belongs to.
.TP
\fBdt_source\fR \fIfile\fR
This command allows the engine to load additional tcl code. The file
being loaded has to be in the same directory as the file the format
engine was loaded from. Any path specified for \fIfile\fR is ignored.
.RE
.TP
Expander commands
All of the commands below are methods of the expander object (without
the prefix \fBex_\fR) handling the input. Their arguments and
results are described in \fBexpander(n)\fR.
.RS
.TP
\fBex_cappend\fR
.TP
\fBex_cget\fR
.TP
\fBex_cis\fR
.TP
\fBex_cname\fR
.TP
\fBex_cpop\fR
.TP
\fBex_cpush\fR
.TP
\fBex_cset\fR
.TP
\fBex_lb\fR
.TP
\fBex_rb\fR
.RE
.TP
_common.tcl commands
Any engine loading (\fBdt_source\fR) the file "\fI_common.tcl\fR" has
default implementations of the \fBfmt_\fR commands explicitly
listed in this document, and can additionally use
.RS
.TP
\fBc_inpass\fR
Returns the id of the pass currently executing
.TP
\fBc_begin\fR
Use this to mark that processing of the text after \fBmanpage_begin\fR
has begun.
.TP
\fBc_begun\fR
Checks the flag set by \fBc_begin\fR.
.TP
\fBc_set_module\fR \fItext\fR
Remember \fItext\fR as module information.
.TP
\fBc_get_module\fR
Retrieve module information stored by \fBc_set_module\fR.
.TP
\fBc_set_title\fR \fItext\fR
Remember \fItext\fR as title.
.TP
\fBc_get_title\fR
Retrieve title stored by \fBc_set_title\fR.
.TP
\fBc_provenance\fR
Returns a string describing how the input was processed.
.TP
\fBc_pass\fR \fIpass proc arguments body\fR
Define a procedure which is valid when pass \fIpass\fR of the engine
is executed.
.TP
\fBc_holdBuffers\fR \fIargs\fR
Define one or more buffers for holding data between passes.
.TP
\fBc_hold\fR \fIbuffer entry\fR
Add an entry to the specified buffer. The buffer has to be defined by
an earlier invocation of the command \fBc_holdBuffers\fR.
.TP
\fBc_held\fR \fIbuffer\fR
Retrieves the contents of the specified buffer. The buffer is empty
afterwards. All entries in the buffer are joined by newlines.
.TP
\fBc_cnext\fR
Increment the counter and return its current value.
.TP
\fBc_cinit\fR
Push the current counter on the stack and reinitialize the counter to zero.
.TP
\fBc_creset\fR
Reinitialize the counter with the value on the counter stack and
remove that value from the stack.
.TP
\fBNOP\fR \fIargs\fR
Do nothing command. Can be used in conjunction with \fBc_pass\fR to
visibly declare in which passes a formatting command has nothing to do.
.TP
\fBNYI\fR ?\fImessage\fR?
Throws the error "\fImessage\fR \fBNot yet implemented\fR".
.TP
\fBc_sectionId\fR \fIname\fR
Remembers the name of the section for later cross-referencing
.TP
\fBc_possibleReference\fR \fItext gi\fR
Checks if a section is available for \fItext\fR. This command is
currently tuned for use by engines like HTML and TMML.
.RE
.RE
.SH "SEE ALSO"
expander(n), format(n), mpexpand(n)
.SH "KEYWORDS"
manpage, interface, TMML, HTML, nroff, conversion, markup
.SH "COPYRIGHT"
.nf
Copyright (c) 2002 Andreas Kupries <andreas_kupries@users.sourceforge.net>
Copyright (c) 2003 Andreas Kupries <andreas_kupries@users.sourceforge.net>
