'\"
'\" Generated from file '../modules/log/logger.man' by tcllib/doctools with format 'nroff'
'\"
'\" -*- tcl -*- doctools manpage
'\" $Id: logger.man,v 1.2 2003/01/19 07:58:44 andreas_kupries Exp $
.TH "logger" n 0.1.0 log "Object Oriented logging facility"
.BS
.SH "NAME"
logger \- System to control logging of events.
.SH "SYNOPSIS"
package require \fBTcl  8\fR
.sp
package require \fBlogger  ?0.1.0?\fR
.sp
\fBlogger::init\fR \fIservice\fR\fR
.sp
\fBlogger::services\fR \fR
.sp
\fBlogger::enable\fR \fIlevel\fR\fR
.sp
\fBlogger::disable\fR \fIlevel\fR\fR
.sp
\fB${log}::debug\fR \fImessage\fR\fR
.sp
\fB${log}::info\fR \fImessage\fR\fR
.sp
\fB${log}::notice\fR \fImessage\fR\fR
.sp
\fB${log}::warn\fR \fImessage\fR\fR
.sp
\fB${log}::error\fR \fImessage\fR\fR
.sp
\fB${log}::critical\fR \fImessage\fR\fR
.sp
\fB${log}::enable\fR \fIlevel\fR\fR
.sp
\fB${log}::disable\fR \fIlevel\fR\fR
.sp
\fB${log}::logproc\fR \fIlevel\fR \fIargname\fR \fIbody\fR\fR
.sp
\fB${log}::services\fR \fR
.sp
\fB${log}::delete\fR \fR
.sp
.BE
.SH "DESCRIPTION"
.PP
The \fBlogger\fR package provides a flexible system for logging messages
from different services, at priority levels, with different commands.
.PP
To begin using the logger package, we do the following:
.PP
.nf
    package require logger
    set log [logger::init myservice]
    ${log}::notice "Initialized myservice logging"

    ... code ...

    ${log}::notice "Ending myservice logging"
    ${log}::destroy
.fi
.PP
In the above code, after the package is loaded, the following things
happen:
.TP
\fBlogger::init\fR \fIservice\fR\fR
Initializes the service \fIservice\fR for logging.  The service names
are actually Tcl namespace names, so they are seperated with '::'.
When a logger service is initalized, it "inherits" properties from its
parents.  For instance, if there were a service \fIfoo\fR, and
we did a \fBlogger::init\fR \fIfoo::bar\fR (to create a \fIbar\fR
service underneath \fIfoo\fR), \fIbar\fR would copy the current
configuration of the \fIfoo\fR service, although it would of
course, also be possible to then seperately configure \fIbar\fR.
.TP
\fBlogger::services\fR \fR
Returns a list of all the available services.
.TP
\fBlogger::enable\fR \fIlevel\fR\fR
Globally enables logging at or "above" the given level.  Levels are
\fBdebug\fR, \fBinfo\fR, \fBnotice\fR, \fBwarn\fR, \fBerror\fR,
\fBcritical\fR.
.TP
\fBlogger::disable\fR \fIlevel\fR\fR
Globally disables logging at or "below" the given level.  Levels are
those listed above.
.TP
\fB${log}::debug\fR \fImessage\fR\fR
.TP
\fB${log}::info\fR \fImessage\fR\fR
.TP
\fB${log}::notice\fR \fImessage\fR\fR
.TP
\fB${log}::warn\fR \fImessage\fR\fR
.TP
\fB${log}::error\fR \fImessage\fR\fR
.TP
\fB${log}::critical\fR \fImessage\fR\fR
These are the commands called to actually log a message about an
event.  \fB${log}\fR is the variable obtained from \fBlogger::init\fR.
.TP
\fB${log}::enable\fR \fIlevel\fR\fR
Enable logging, in the service referenced by \fB${log}\fR, and its
children.
.TP
\fB${log}::disable\fR \fIlevel\fR\fR
Disable logging, in the service referenced by \fB${log}\fR, and
its children.
.TP
\fB${log}::logproc\fR \fIlevel\fR \fIargname\fR \fIbody\fR\fR
This is a command to define a command that will perform the actual
logging for a given level.  The logger package ships with default
commands for all log levels, but with \fBlogproc\fR it is possible to
replace them with custom code.  This would let you send your logs over
the network, to a database, or anything else.  For example:
.nf
    ${log}::logproc notice txt {
	puts $netlog "Notice: $txt"
    }
.fi
.TP
\fB${log}::services\fR \fR
Returns a list of all the registered logging services.
.TP
\fB${log}::delete\fR \fR
This command deletes a particular logging service, and its children.
You must call this to clean up the resources used by a service.
.SH "IMPLEMENTATION"
The logger package is implemented in such a way as to optimize (for
Tcl 8.4 and newer) log procedures which are disabled.  They are
aliased to a proc which has no body, which is compiled to a no op in
bytecode.  This should make the peformance hit minimal.  If you really
want to pull out all the stops, you can replace the ${log} token in
your code with the actual namespace and command (${log}::warn becomes
::logger::tree::myservice::warn), so that no variable lookup is done.
This puts the performance of disabled logger commands very close to no
logging at all.
.PP
The "object orientation" is done through a hierarchy of namespaces.
Using an actual object oriented system would probably be a better way
of doing things, or at least provide for a cleaner implementation.
.PP
The service "object orientation" is done with namespaces.
.SH "KEYWORDS"
