#-----------------------------------------------------------------------------
#                            CarthaGene
#
# Martin Bouchez, Patrick Chabrier, Christine Gaspin and Thomas Schiex. 
# Unite de Biometrie et Intelligence Artificielle, Toulouse, France
#
#  Copyright (C) 1997 INRA (Institut National de la Recherche Agronomique).
#
# contrib.tcl,v 1.2 2004/06/30 07:43:54 degivry Exp
#
#    This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
# Module        : cgscript
# Projet        : CartaGene
#
# Description : features implemented in tcl only
#
# Divers : 
#-----------------------------------------------------------------------------
#-INRA---------------------Station de Biomtrie et d'Intelligence Artificielle
#-----------------------------------------------------------------------------

#
# features implemented in tcl
#

# print the best map
proc bestprint {} {
    maprint [lindex [lindex [heapget k 1] 0 ] 0]
}

# print in detail the best map
proc bestprintd {} {
    maprintd [lindex [lindex [heapget k 1] 0 ] 0]
}

proc mrkdel { mrk } {
	# to get the id of the marker
	set id [mrkid $mrk]
	# to get the selection
	set sel [mrkselget]
	# to remove
	set pos [lsearch $sel $id]
	set nsel [lreplace $sel $pos $pos]
	# to set the selection
	mrkselset $nsel
}

# add a set of markers from their loci ids
proc mrkadd {args} {
     set loc "[mrkselget] "
     append loc $args
     mrkselset $loc
}

# remove a set of markers from their loci ids 

proc mrksub {args} {
     set loc " [mrkselget] "
     foreach item $args {
        regsub " $item " $loc " " res
        set loc $res
      }
      mrkselset $loc
}

# returns the ids of several markers
proc mrkids {args} {
     set args [join $args]
     foreach item $args {
     lappend loc [mrkid $item]
}
return $loc	    
}

#returns the names of several markers ids

proc mrknames {args} {
     set args [join $args]
     foreach item $args {
     lappend loc [mrkname $item]
}
return $loc	    
}

proc lkh1 {name nbrun} {
    # create LKH parameter file
    if [catch {open "$name.lkh" w} fid] {
	puts stderr "Cannot open $name.lkh : $fid"
	exit
    }
    puts $fid "PROBLEM_FILE = $name.tsp"
    puts $fid "TOUR_FILE = $name.sol"
    puts $fid "TRACE_LEVEL = 1"
    puts $fid "RUNS = $nbrun"
    close $fid
    # run LKH
    set thepath [lindex $::auto_path [lsearch $::auto_path *dev]]
    set res [exec echo $name.lkh | LKH | awk -f $thepath/cgscript/lkh.awk]
    puts $res
    # create initial marker selection file
    if [catch {open "$name.ord" w} fid] {
	puts stderr "Cannot open $name.ord : $fid"
	exit
    }
    foreach id [mrkselget] {
	puts $fid "$id"
    }
    close $fid
    # create a new marker order from the TSP solution
    set order [exec awk -f $thepath/cgscript/lkh2cg.awk $name.ord $name.sol | grep mrkselset]
    if [catch {open "$name.tcl" w} fid] {
	puts stderr "Cannot open $name.tcl : $fid"
	exit
    }
    puts $fid $order
    close $fid
    # read the order
    source $name.tcl
    # evaluate the solution in multipoint
    puts "and, in multi-points:"
    sem
    puts ""
    return $res
}

proc concorde1 {name} {
    # run Concorde
    set thepath [lindex $::auto_path [lsearch $::auto_path *dev]]
    set res [exec concorde $name.tsp | awk -f $thepath/cgscript/concorde.awk]
    puts $res
    # create initial marker selection file
    if [catch {open "$name.ord" w} fid] {
	puts stderr "Cannot open $name.ord : $fid"
	exit
    }
    foreach id [mrkselget] {
	puts $fid "$id"
    }
    close $fid
    # create modified solution file
    if [catch {open "$name.sol2" w} fid] {
	puts stderr "Cannot open $name.sol2 : $fid"
	exit
    }
    set solution [exec sed -f $thepath/cgscript/csol.sed $name.sol]
    puts $fid $solution
    close $fid
    # create a new marker order from the TSP solution
    set order [exec awk -f $thepath/cgscript/concorde2cg.awk $name.ord $name.sol2 | grep mrkselset]
    if [catch {open "$name.tcl" w} fid] {
	puts stderr "Cannot open $name.tcl : $fid"
	exit
    }
    puts $fid $order
    close $fid
    # read the order
    source $name.tcl
    # evaluate the solution in multipoint
    puts "and, in multi-points:"
    sem
    puts ""
    return $res
}

proc flkh {nbrun} {
    # create TSP files
    cg2tsp tmp.tsp
    set initorder [ mrkselget ]
    puts "-----------------------------------------------------------"
    puts "--- Convert to TSP without taking into account unknowns ---"
    lkh1 "tmp" $nbrun
    puts "---------------------------------------------------------------------------"
    puts "--- Convert to TSP with normalization (by taking into account unknowns) ---"
    mrkselset $initorder
    lkh1 "normtmp" $nbrun
    puts "-------------------------------------------------------------------------------"
    puts "--- Convert to TSP with LOD as cost matrix ---"
    mrkselset $initorder
    lkh1 "lodtmp" $nbrun
    puts "----------------------------------------------------------------------------------"
    puts "--- Convert to TSP with Haldane / Ray distance matrix ---"
    mrkselset $initorder
    lkh1 "disttmp" $nbrun

    mrkselset $initorder
    return
}

proc fconcorde {} {
    # create TSP files
    cg2tsp tmp.tsp
    set initorder [ mrkselget ]
    concorde1 "tmp"
    mrkselset $initorder
    return
}
